package lib.ui.reactive;

import lib.ui.Dimensions;
import util.ObjMerge;
import lib.Pos;
import util.Observable;
import lib.Color;
import lib.Vec.Vec2;

using tink.CoreApi;

typedef TextElementArgs = {
	public var ?text:Observable<String>;
	public var ?simpleText: String;
	public var ?bg:Color;
	public var ?fg:Color;
	public var ?padding: Dimensions;
	public var ?margin: Dimensions;
}

class TextElement extends UIElement {
	private var args:TextElementArgs;

	public function new(args: TextElementArgs,events: UIEvents = null) {
		super(events);
		this.args = args;

		if (args.text != null) {
			args.text.subscribe(value -> {
				this.changedTrigger.trigger(null);
			});
		}else if (args.simpleText == null) {
			throw new Error("text or simpleText must be set");
		}
	}

	private function getText() {
		if (args.text != null) {
			return args.text.get();
		} else {
			return args.simpleText;
		}
	}

	public function render(bounds:Vec2<Int>,offset: Pos):Canvas {
		var rtn = new Canvas();

		var fullText = getText();

		var writePoint: Pos = {x: 0,y: 0};

		for (pos in 0...fullText.length) {
			var char = fullText.charAt(pos);

			if (char == "\n"){
				writePoint = {x: 0, y: writePoint.y + 1};
				continue;
			}

			if (writePoint.y >= bounds.y) {
				break;
			}

			if (writePoint.x >= bounds.x) {
				writePoint = {x: 0, y: writePoint.y + 1};
			}

			rtn.set(writePoint, {char: char,textColor: this.args.fg, bg: this.args.bg});

			writePoint = {x: writePoint.x + 1, y: writePoint.y};
		}



		return UIElement.applyPaddignAndMargin(rtn, this.args.padding, this.args.margin);
	}
}
